<?php


class BF_Fields_To_Gutenberg {

	/**
	 * @var array
	 */
	protected $fields = [];

	/**
	 * @var array
	 */
	protected $stds = [];


	/**
	 * BF_Fields_To_Gutenberg constructor.
	 *
	 * @param array $fields
	 * @param array $stds
	 *
	 * @since 3.9.0
	 */
	public function __construct( array $fields = [], $stds = [] ) {

		$this->load_fields( $fields );
		$this->load_stds( $stds );
	}


	/**
	 * Set standard BF fields array.
	 *
	 * @param array $fields
	 *
	 * @since 3.9.0
	 */
	public function load_fields( array $fields ) {

		foreach ( $fields as $id => $field ) {

			if ( isset( $field['override-gutenberg'] ) ) {

				$fields[ $id ] = array_merge( $field, $field['override-gutenberg'] );

				unset( $fields[ $id ]['override-gutenberg'] );
			}
		}

		$this->fields = $fields;
	}


	/**
	 * Set fields default std value.
	 *
	 * @param array $stds
	 *
	 * @since 3.9.0
	 */
	public function load_stds( array $stds ) {

		$this->stds = $stds;
	}


	/**
	 * Get BF fields array.
	 *
	 * @since 3.9.0
	 * @return array
	 */
	public function fields() {

		return $this->fields;
	}


	/**
	 * Transform fields to gutenberg format.
	 *
	 * @param BF_Gutenberg_Field_Transformer|null $parent_field parent object if exists
	 *
	 * @since 3.9.0
	 *
	 * @return array
	 */
	public function transform( BF_Gutenberg_Field_Transformer $parent_field = null ) {

		$container = [];
		$results   = &$container;

		$tab_started = false;
		$iteration   = 0;

		foreach ( $this->fields as $field ) {

			if ( empty( $field['type'] ) ) {

				continue;
			}

			// replace wp_editor with textarea
			if ( 'wp_editor' === $field['type'] ) {
				$field['type'] = 'textarea';
			}

			if ( ! isset( $field['std'] ) && isset( $field['id'] ) ) {

				$id = $field['id'];

				if ( isset( $this->stds[ $id ] ) ) {

					$field['std'] = $this->stds[ $id ];
				}
			}

			if ( isset( $field['std'] ) ) {

				$field['default'] = $field['std'];

				unset( $field['std'] );
			}

			$field_type = $field['type'];

			if ( 'group_close' === $field_type || 'panel_end' === $field_type ) {

				$container = &$results;
				continue;
			}

			if ( 'tab' === $field_type && $tab_started ) {

				$tab_started = false;
				$container   = &$results;
			}

			$factory = $this->factory( $field, $results );

			if ( ! $factory ) {
				continue;
			}

			$this->transform_field( $container, $factory, ++ $iteration, $parent_field );


			if ( 'group' === $field_type || 'tab' === $field_type || 'panel' === $field_type ) {

				$tab_started = true;

				end( $container );
				$key = key( $container );

//				$results   = &$container;
				$container = &$container[ $key ]['children'];
			}
		}

		return $results;
	}


	public function list_attributes() {

		$parent  = [];
		$results = [];

		foreach ( $this->fields as $field ) {

			if ( empty( $field['type'] ) ) {
				continue;
			}

			$factory = $this->factory( $field, $results );

			if ( ! $factory ) {
				continue;
			}

			$the_attribute = $factory->the_attribute( $parent );

			if ( $the_attribute ) {

				$results[ $factory->field( 'id' ) ] = $the_attribute;
			}

			if ( 'tab' === $field['type'] ) {

				$parent = $field;
			}
		}

		return $results;
	}


	/**
	 * @param array                          $container
	 * @param BF_Gutenberg_Field_Transformer $field
	 * @param int                            $iteration
	 * @param BF_Gutenberg_Field_Transformer $parent_transformer
	 *
	 * @since 3.9.0
	 * @return bool true on success or false on failure.
	 */
	protected function transform_field( &$container, BF_Gutenberg_Field_Transformer $field, $iteration, BF_Gutenberg_Field_Transformer $parent_transformer = null ) {

		$transformed = $field->transform_field( $iteration );

		if ( ! is_array( $transformed ) ) {

			return false;
		}

		$id   = $field->field( 'id' );
		$data = [
			'id'        => $id,
			'key'       => $id,
			'data_type' => $field->data_type(),
			'component' => $field->component(),
			'args'      => $transformed,
		];

		$attribute = $field->the_attribute();

		if ( $attribute ) {

			$data['attribute'] = $attribute;
		}

		$settings = $field->settings();

		if ( $settings ) {

			$data['args']['_settings'] = $settings;
		}

		$shared_keys = array_intersect_key( $field->field(), [
			'include_blocks' => '',
			'exclude_blocks' => '',
			'only_widgets'   => '',
			'fixed_class'    => '',
			'priority'       => '',
			'action'         => '',
		] );

		if ( $shared_keys ) {

			$data = array_merge( $data, $shared_keys );
		}

		$children = $field->children_items_list();

		if ( $children ) {

			$children_handler = new self( $children );

			$children_transformed = $children_handler->transform( $field );

			if ( $children_transformed ) {

				$data['children'] = $children_transformed;
			}
		}

		if ( $parent_transformer ) {

			$data = $parent_transformer->children_item( $data );
		}

		if ( $field->wrap_section_container ) {

			$container[] = $this->wrap_section_container( $field, $data );

		} else {

			$container[] = $data;
		}

		return true;
	}


	/**
	 * @param BF_Gutenberg_Field_Transformer $field
	 * @param array                          $data
	 *
	 * @return array
	 */
	public function wrap_section_container( $field, $data ) {

		$title = $field->field( 'name' );
		unset( $data['args']['label'] );

		$id = $field->field( 'id' );

		$args = array(
			'type'  => $field->field( 'type' ),
			'label' => $title,
			'id'    => $id,
			'name'  => $id,
		);

		$classes = $field->field( 'container_class' );

		if ( $classes ) {
			$args['container_class'] = $classes;
		}

		$classes = $field->field( 'section_class' );

		if ( $classes ) {
			$args['section_class'] = $classes;
		}

		if ( ! empty( $data['args']['desc'] ) ) {
			$args['description'] = $data['args']['desc'];

			unset( $data['args']['desc'] );
		}

		$show_on = $field->field( 'show_on' );

		if ( $show_on ) {

			if ( ! function_exists( 'bf_show_on_attributes' ) ) {
				require BF_PATH . '/core-deprecated/field-generator/functions.php';
			}

			$args['show_on'] = bf_show_on_settings( $field->field() );
		}

		$props = [
			'data_type' => $field->data_type(),
			'component' => 'section_container',
			'children'  => [ $data ],
			'args'      => $args,
			'key'       => "field_$id",
			'id'        => "field_$id",
		];

		$dynamic_props = $field->dynamic_values_indexes();

		if ( $dynamic_props ) {

			$props['dynamic_props'] = $dynamic_props;
		}

		return $props;
	}

	/**
	 * @param array $field
	 * @param array $results
	 *
	 * @since 3.9.0
	 * @return BF_Gutenberg_Field_Transformer on success or null on failure.
	 */
	public function factory( array $field, array &$results ) {

		if ( empty( $field['type'] ) ) {
			return null;
		}

		switch ( $field['type'] ) {

			case 'group':
			case 'tab':
				$instance = new BF_Gutenberg_Panel_Body();

				break;

			case 'panel':
				$instance = new BF_Gutenberg_BF_Panel();

				break;

			case 'date':
				$instance = new BF_Gutenberg_Date_Time_Picker();

				break;

			case 'color':
				$instance = new BF_Gutenberg_Color_Palette();

				break;

			case 'tabs':
				$instance = new BF_Gutenberg_Tabs();

				break;

			case 'gutenberg':
				$instance = new BF_Gutenberg_Native_Component();

				break;

			case 'repeater':
				$instance = new BF_Gutenberg_BF_Repeater();

				break;

			default:
				$instance = new BF_Gutenberg_Control();

				break;
		}

		$instance->init( $field, $results );

		return $instance;
	}
}
