<?php declare( strict_types=1 );

namespace BetterStudio\Component\Standard\Block\UnitTest;

use BetterStudio\Component\Standard\Block\BlockStorage;

//
use BetterStudio\Component\TestUtil as TestUtil;

class TestBlockStorage extends TestCase {

	/**
	 * @covers BlockStorage::factory
	 */
	public function test_factory_return_null() {

		$this->assertNull(
			BlockStorage::factory( 'wrong-id' ),
			'The factory method must return null when the passed block_id is null.'
		);
	}

	/**
	 * @covers BlockStorage::factory
	 */
	public function test_factory_return_block_exact_instance() {

		$block = new TestUtil\Sample\Block\SampleBlock();
		$this->blocks_set( [
			'block-id' => $block
		] );

		$this->assertSame(
			$block,
			BlockStorage::factory( 'block-id' ),
			'The factory method must return the exact instance of the block'
		);
	}

	/**
	 * @covers BlockStorage::factory
	 */
	public function test_factory_return_new_block_instance() {

		$class_name = TestUtil\Sample\Block\SampleBlock::class;

		$this->blocks_set( [
			'block-id' => $class_name
		] );

		$this->assertInstanceOf(
			$class_name,
			BlockStorage::factory( 'block-id' ),
			'The factory method must create new block instance and return it'
		);
	}

	/**
	 * @covers BlockStorage::exists
	 */
	public function test_exist_method() {

		$this->blocks_set( [
			'block-id' => TestUtil\Sample\Block\SampleBlock::class
		] );

		$this->assertTrue(
			BlockStorage::exists( 'block-id' ),
			'The exists method must return ture when the block id is correct'
		);
		$this->assertFalse(
			BlockStorage::exists( 'wrong-id' ),
			'The exists method must return false when the block id is incorrect'
		);
	}

	/**
	 * @covers BlockStorage::register
	 */
	public function test_register_method() {

		$this->blocks_flush();
		$block = new TestUtil\Sample\Block\SampleBlock();

		$this->assertTrue(
			BlockStorage::register( 'block-id', $block ),
			'The register method must return true when a block was successfully registered'
		);

		$this->assertSame(
			[ 'block-id' => $block ],
			$this->blocks_get(),
			'The register method must fill protected $blocks property on registering a block'
		);
	}

	/**
	 * @covers BlockStorage::register
	 */
	public function test_register_method_return_false() {

		$block = new TestUtil\Sample\Block\SampleBlock();

		BlockStorage::register( 'block-id', $block );

		$this->assertFalse(
			BlockStorage::register( 'block-id', $block ),
			'The register method should return false when it\'s already registered'
		);
	}

	/**
	 * @covers BlockStorage::flush
	 */
	public function test_flush_method() {

		$fake_data = [
			'first',
			'Second',
			3
		];
		$this->blocks_set( $fake_data );

		BlockStorage::flush();

		$this->assertSame(
			[],
			$this->blocks_get(),
			'The flush method must set an empty array to $blocks property.'
		);
	}

	/**
	 * @covers BlockStorage::blocks()
	 */
	public function test_blocks_method_return_all_blocks() {

		$fake_data = [
			'first',
			'Second',
			'Third'
		];
		$this->blocks_set( $fake_data );

		$this->assertSame(
			$fake_data,
			BlockStorage::blocks(),
			'The blocks method must return the exact $blocks property.'
		);
	}

	/**
	 * @covers BlockStorage::blocks()
	 */
	public function test_blocks_method_apply_filters() {

		$fake_data = [
			'first',
			'Second',
			'Third',
			4
		];
		$this->blocks_set( $fake_data );

		$this->expectFilter(
			'better-studio/blocks/list',
			$fake_data
		);

		BlockStorage::blocks();
	}

	///
	/// Internal methods
	///

	/**
	 * Fill the protected 'blocks' property.
	 *
	 * @param mixed $data
	 *
	 * @return bool true on success or false on failure.
	 */
	protected function blocks_set( $data ) {

		return $this->property_set( BlockStorage::class, 'blocks', $data );
	}

	/**
	 * Get the protected 'blocks' property.
	 *
	 * @param mixed $data
	 *
	 * @return bool true on success or false on failure.
	 */
	protected function blocks_get() {

		$this->property_get( BlockStorage::class, 'blocks', $result );

		return $result;
	}

	/**
	 * Clear protected 'blocks' property value.
	 */
	protected function blocks_flush() {

		$this->blocks_set( [] );
	}
}
