<?php declare( strict_types=1 );

namespace BetterStudio\Component\Integration\Control\UnitTest;


use BetterStudio\Component\Integration\Control as Module;
use BetterStudio\Component\Standard\{
	Control as ControlStandard
};

//
use BetterStudio\Component\TestUtil;

class TestControlsStorage extends TestUtil\TestCase {

	/**
	 * @covers Module\ControlsStorage::factory
	 */
	public function test_factory_return_null() {

		$this->assertNull(
			Module\ControlsStorage::factory( 'wrong-id' ),
			'The factory method must return null when the passed control_type is null.'
		);
	}

	/**
	 * @covers Module\ControlsStorage::factory
	 */
	public function test_factory_return_control_exact_instance() {

		$control = new TestUtil\Sample\Control\Samplecontrol( [] );
		$this->controls_set( [
			'control-id' => $control
		] );

		$this->assertSame(
			$control,
			Module\ControlsStorage::factory( 'control-id' ),
			'The factory method must return the exact instance of the control'
		);
	}

	/**
	 * @covers Module\ControlsStorage::factory
	 */
	public function test_factory_must_load_the_options() {

		$control = new TestUtil\Sample\Control\Samplecontrol( [] );
		$this->controls_set( [
			'control-id'   => $control,
			'control-id-2' => TestUtil\Sample\Control\Samplecontrol::class,
		] );

		$options   = [
			'label' => 'x',
			'data'  => 'here',
		];
		$control_1 = Module\ControlsStorage::factory( 'control-id', $options );
		$control_2 = Module\ControlsStorage::factory( 'control-id-2', $options );

		$this->assertSame(
			$options,
			$this->get_control_options( $control_1 )
		);

		$this->assertSame(
			$options,
			$this->get_control_options( $control_2 )
		);
	}

	/**
	 * @covers Module\ControlsStorage::factory
	 */
	public function test_factory_return_new_control_instance() {

		$class_name = TestUtil\Sample\Control\Samplecontrol::class;

		$this->controls_set( [
			'control-id' => $class_name
		] );

		$this->assertInstanceOf(
			$class_name,
			Module\ControlsStorage::factory( 'control-id' ),
			'The factory method must create new control instance and return it'
		);
	}


	/**
	 * @covers Module\ControlsStorage::exists
	 */
	public function test_exist_method() {

		$this->controls_set( [
			'control-id' => TestUtil\Sample\Control\Samplecontrol::class
		] );

		$this->assertTrue(
			Module\ControlsStorage::exists( 'control-id' ),
			'The exists method must return ture when the control id is correct'
		);
		$this->assertFalse(
			Module\ControlsStorage::exists( 'wrong-id' ),
			'The exists method must return false when the control id is incorrect'
		);
	}

	/**
	 * @covers Module\ControlsStorage::register
	 */
	public function test_register_method() {

		$this->controls_flush();
		$control = new TestUtil\Sample\Control\Samplecontrol( [] );

		$this->assertTrue(
			Module\ControlsStorage::register( 'control-id', $control ),
			'The register method must return true when a control was successfully registered'
		);

		$this->assertSame(
			[ 'control-id' => $control ],
			$this->controls_get(),
			'The register method must fill protected $controls property on registering a control'
		);
	}

	/**
	 * @covers Module\ControlsStorage::register
	 */
	public function test_register_method_return_false() {

		$control = new TestUtil\Sample\Control\Samplecontrol( [] );

		Module\ControlsStorage::register( 'control-id', $control );

		$this->assertFalse(
			Module\ControlsStorage::register( 'control-id', $control ),
			'The register method should return false when it\'s already registered'
		);
	}

	/**
	 * @covers Module\ControlsStorage::flush
	 */
	public function test_flush_method() {

		$fake_data = [
			'first1',
			'Second',
			3
		];
		$this->controls_set( $fake_data );

		Module\ControlsStorage::flush();

		$this->assertSame(
			[],
			$this->controls_get(),
			'The flush method must set an empty array to $controls property.'
		);
	}

	/**
	 * @covers Module\ControlsStorage::controls()
	 */
	public function test_controls_method_return_all_controls() {

		$fake_data = [
			'first2',
			'Second',
			'Third'
		];
		$this->controls_set( $fake_data );

		$this->assertSame(
			$fake_data,
			Module\ControlsStorage::controls(),
			'The controls method must return the exact $controls property.'
		);
	}

	/**
	 * @covers Module\ControlsStorage::controls()
	 */
	public function test_controls_method_apply_filters() {

		$fake_data = [
			'first3',
			'Second',
			'Third',
			4
		];
		$this->controls_set( $fake_data );

		$this->expectFilter(
			'better-studio/controls/list',
			$fake_data
		);

		Module\ControlsStorage::controls();
	}

	///
	/// Internal methods
	///

	/**
	 * Fill the protected 'controls' property.
	 *
	 * @param mixed $data
	 *
	 * @return bool true on success or false on failure.
	 */
	protected function controls_set( $data ) {

		return $this->property_set( Module\ControlsStorage::class, 'controls', $data );
	}

	/**
	 * Get the protected 'controls' property.
	 *
	 * @return bool true on success or false on failure.
	 */
	protected function controls_get() {

		$this->property_get( Module\ControlsStorage::class, 'controls', $result );

		return $result;
	}

	/**
	 * Clear protected 'controls' property value.
	 */
	protected function controls_flush() {

		$this->controls_set( [] );
	}


	/**
	 * @param object the control instance $control
	 */
	protected function get_control_options( $control ) {

		$this->property_get( $control, 'options', $value );

		return $value;
	}
}
