<?php

namespace BetterStudio\Utils\ArrayUtil;

use ErrorException;
use ArrayIterator;

/**
 * Collection Base class.
 *
 * @since   1.0.0
 * @package BetterStudio/Utils/ArrayUtil
 */
abstract class CollectionBase implements \Countable, \ArrayAccess, \IteratorAggregate {

	/**
	 * Load the array into the collection.
	 *
	 * @since 1.0.0
	 * @var array
	 *
	 */
	protected $data;

	/**
	 * Get a value by index.
	 *
	 * @param string $index
	 *
	 * @since 1.0.0
	 * @return mixed null on failure.
	 */
	public function get( string $index ) {

		if ( strpos( $index, '.' ) === false ) {

			return $this->data[ $index ] ?? null;
		}

		$indexes = explode( '.', $index );
		$ref     = &$this->data;

		foreach ( $indexes as $c_index ) {

			if ( isset( $ref[ $c_index ] ) ) {

				$ref = &$ref[ $c_index ];
			} else {

				return null;
			}
		}

		return $ref;
	}

	/**
	 * Store value in given index.
	 *
	 * @param string $index
	 * @param mixed  $value
	 *
	 * @since 1.0.0
	 */
	public function set( string $index, $value ): void {

		if ( strpos( $index, '.' ) === false ) {

			$this->data[ $index ] = $value;

			return;
		}

		$indexes = explode( '.', $index );
		$ref     = &$this->data;

		foreach ( $indexes as $c_index ) {

			if ( ! isset( $ref[ $c_index ] ) ) {

				$ref[ $c_index ] = array();
			}

			$ref = &$ref[ $c_index ];
		}

		$ref = $value;
	}

	/**
	 * Load default values.
	 *
	 * @param iterable $defaults
	 *
	 * @since 1.1.0
	 */
	public function defaults( iterable $defaults ): void {

		foreach ( $defaults as $key => $default_value ) {

			if ( ! isset( $this->data[ $key ] ) ) {

				$this->set($key, $default_value);
			}
		}
	}

	/**
	 * Remove a key.
	 *
	 * @param int|string $index
	 *
	 * @since     1.1.0
	 * @copyright https://stackoverflow.com/a/46466325
	 */
	public function remove( $index ): void {

		$keys    = explode( '.', $index );
		$pointer = &$this->data;
		//
		while ( ( $current = array_shift( $keys ) ) && ( count( $keys ) > 0 ) ) {
			// if some key is missing all the sub-keys will be already unset
			if ( ! array_key_exists( $current, $pointer ) ) {
				// is already unset somewhere along the way
				return;
			}
			// set pointer to new, deeper level
			// called for all but last key
			$pointer = &$pointer[ $current ];
		}
		// handles empty input string
		if ( $current ) {
			// we finally unset what we wanted
			unset( $pointer[ $current ] );
		}
	}


	/**
	 * Get all stored data as an array.
	 *
	 * @since 1.0.0
	 * @return array
	 */
	public function to_array(): array {

		if ( $this->data instanceof self ) {

			return $this->data->to_array();
		}

		return array_map( static function ( $item ) {

			return $item instanceof self ? $item->to_array() : $item;

		}, $this->data ?? [] );
	}

	/**
	 * Count items.
	 *
	 * @since 1.0.2
	 * @return int
	 */
	public function count() {

		return count( $this->to_array() ) ?? 0;
	}

	/**
	 * @inheritDoc
	 *
	 * @since 1.0.2
	 * @return bool
	 */
	public function offsetExists( $offset ) {

		if ( strpos( $offset, '.' ) === false ) {

			return isset( $this->data[ $offset ] );
		}

		return $this->get( $offset ) !== null;
	}

	/**
	 * @inheritDoc
	 *
	 * @since 1.0.2
	 * @return bool
	 */
	public function offsetGet( $offset ) {

		return $this->get( $offset );
	}

	/**
	 * @param mixed $offset
	 * @param mixed $value
	 *
	 * @throws ErrorException
	 * @since 1.0.2
	 */
	public function offsetSet( $offset, $value ) {

		throw new ErrorException( __CLASS__ . ' is read only' );
	}

	/**
	 * @param mixed $offset
	 *
	 * @throws ErrorException
	 * @since 1.0.2
	 */
	public function offsetUnset( $offset ) {

		throw new ErrorException( __CLASS__ . ' is read only' );
	}

	/**
	 * @inheritDoc
	 *
	 * @since 1.0.2
	 * @return ArrayIterator
	 */
	public function getIterator() {

		return new ArrayIterator( $this->data??[] );
	}
}
